🔬This is a nightly-only experimental API. (ptr_metadata#81513)
This item is validated for IEC 61508 (SIL 2) and ISO 26262 (ASIL B).
The metadata for a Dyn = dyn SomeTrait trait object type.
It is a pointer to a vtable (virtual call table)
that represents all the necessary information
to manipulate the concrete type stored inside a trait object.
The vtable notably contains:
type size
type alignment
a pointer to the type’s drop_in_place impl (may be a no-op for plain-old-data)
pointers to all the methods for the type’s implementation of the trait
Note that the first three are special because they’re necessary to allocate, drop,
and deallocate any trait object.
It is possible to name this struct with a type parameter that is not a dyn trait object
(for example DynMetadata<u64>) but not to obtain a meaningful value of that struct.
Note that while this type implements PartialEq, comparing vtable pointers is unreliable:
pointers to vtables of the same type for the same trait can compare inequal (because vtables are
duplicated in multiple codegen units), and pointers to vtables of different types/traits can
compare equal (since identical vtables can be deduplicated within a codegen unit).